/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using NAnt.NUnit2.Tasks;
using TopCoder.Test.Framework.Servers;
using NAnt.Core.Attributes;
using System.Collections;
using System;
using NAnt.Core;

namespace TopCoder.Test.Framework
{

    /// <summary>
    /// <p>
    /// This class derives from NUnit2Task.
    /// This class represents a NAnt task  that extends the optional NUnit2 task to provide support for
    /// testing using this Component.
    /// This is the main NAnt task of this component
    /// </p>
    /// <p>
    /// Supported attributes: none additional to the supported by "nunit2" task.<br/>
    /// Nested  XML elements: "server" -  should be at least one.<br/>
    /// </p>
    /// <p>
    /// This class is mutable, i.e. not thread-safe.
    /// </p>
    /// </summary>
    ///
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TaskName("serverapptest")]
    public class ServerAppTestTask : NUnit2Task
    {

        /// <summary>
        /// <p>
        /// This variable stores the array of server elements.
        /// All elements of the array should be ServerElement instances.
        /// It is accessed through the "Servers" property.
        /// It represents an array of nested "server" XML elements,
        /// describing a particular servers to be started/initialized
        /// while running tests.
        /// This array will is used in ExecuteTask() method.
        /// The "serverapptest" NAnt task should include
        /// at least one nested "server" XML element.
        /// Initial value is null.
        /// </p>
        /// </summary>
        private ServerElement[] servers = null;

        /// <summary>
        /// This property is used to access the "servers" variable.
        /// The "servers" variable stores the array of server elements.
        /// All elements of the array should be  ServerElement instances.
        /// It represents an array of nested "server" XML elements,
        /// describing particular servers to be started/initialized
        /// while running tests.
        /// </summary>
        ///
        /// <value>The array of nested "server" XML elements describing a particular servers to be
        /// started/initialized while running tests.</value>
        [BuildElementArray("server", ElementType=typeof(ServerElement), Required=true)]
        public ServerElement[] Servers
        {
            get
            {
                return servers;
            }
            set
            {
                servers = value;
            }
        }

        /// <summary>
        /// <p>
        /// Creates an instance of class.
        /// </p>
        /// </summary>
        public ServerAppTestTask() : base()
        {
            // Just call constructor of superclass
        }

        /// <summary>
        /// <p>
        /// Executes the task. At first, all servers will be initialized if needed.
        /// Then the main NUnit task is executed. Servers will be stopped if needed, regardless whether
        /// test passed or not.
        /// </p>
        /// </summary>
        ///
        /// <exception cref="ServerTaskException">If any error happens (this exception wll contain also
        /// the array of inner exceptions).</exception>
        protected override void ExecuteTask()
        {

            StartUpServers();

            ArrayList exceptions = new ArrayList();
            // Run the NUnit test task
            try
            {
                base.ExecuteTask();
            }
            catch (BuildException be)
            {
                exceptions.Add(be);
            }

            // Deinitialize/shutdown the servers, even if tests failed
            foreach (ServerElement element in this.servers)
            {
                Server server = null;
                try
                {
                    server = element.GetServerInstance();
                }
                catch (ServerException se)
                {
                    exceptions.Add(se);
                }
                if (server != null)
                {
                    // clean up if necessary, exception is added to the list
                    if (server.InitServer)
                    {
                        try
                        {
                            server.CleanUpData();
                        }
                        catch (ServerException se)
                        {
                            exceptions.Add(se);
                        }
                    }

                    // shut down if necessary, exception is added to the list
                    if (server.StartServer)
                    {
                        try
                        {
                            server.ShutDown();
                        }
                        catch (ServerException se)
                        {
                            exceptions.Add(se);
                        }
                    }
                }

            }

            // if there are any exceptions, wrap them to the ServerTaskException
            if (exceptions.Count > 0)
            {
                throw new ServerTaskException("Failed to exectue the task",
                    (Exception[]) exceptions.ToArray(typeof(Exception)));
            }

        }

        /// <summary>
        /// StartUp the servers.
        /// </summary>
        /// <exception cref="ServerTaskException">If any error happens (this exception wll contain also
        /// the array of inner exceptions).</exception>
        private void StartUpServers()
        {
            // Startup/Initialize the servers, and all exceptions are wrapped into a ServerTaskException
            ArrayList exceptions = new ArrayList();
            foreach (ServerElement element in this.servers)
            {
                bool initialized = false;
                Server server = null;
                try
                {
                    server = element.GetServerInstance();
                    if (server.InitServer)
                    {
                        server.InitData();
                        initialized = true;
                    }
                    if (server.StartServer)
                    {
                        server.StartUp();
                    }
                }
                catch (ServerException se)
                {
                    if (initialized)
                    {
                        try
                        {
                            server.CleanUpData();
                        }
                        catch (ServerException)
                        {
                            // ignoro this
                        }
                    }
                    exceptions.Add(se);
                }

            }

            // if there are any exceptions, wrap them to the ServerTaskException
            if (exceptions.Count > 0)
            {
                throw new ServerTaskException("Failed to start up the servers.",
                    (Exception[]) exceptions.ToArray(typeof(Exception)));
            }
        }
    }
}
